//-------------------------------------------------------------------------------------------------
// <copyright file="BurnMsiPackage.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

class CBurnMsiPackage : public CBurnPackageBase<IBurnMsiPackage, IBurnPackageTestable>
{
public: // IBurnMsiPackage
    //
    // GetProductCode - gets the product code.
    //
    STDMETHODIMP GetProductCode(
        __out LPWSTR* psczProductCode
        )
    {
        HRESULT hr = StrAllocString(psczProductCode, m_sczProductCode, 0);
        ExitOnFailure(hr, "Failed to copy product code.");

    LExit:
        return hr;
    }


public: // internals
    //
    // SetPayloadCacheRoot - sets the cache root path for each payload.
    //
    STDMETHODIMP SetPayloadCacheRoot()
    {
        HRESULT hr = S_OK;
        LPWSTR sczLocalAppData = NULL;
        LPWSTR sczCacheRoot = NULL;
        WCHAR wzVersion[36] = {0};

        hr = PathGetKnownFolder(m_fPerMachine ? CSIDL_COMMON_APPDATA : CSIDL_LOCAL_APPDATA, &sczLocalAppData);
        ExitOnFailure1(hr, "Failed to find local %s appdata directory.", m_fPerMachine ? "per-machine" : "per-user");

        hr = ::StringCchPrintfW(wzVersion, countof(wzVersion), L"%I64d.%I64d.%I64d.%I64d",
            (m_qwVersion & 0xffff000000000000) >> 48, 
            (m_qwVersion & 0x0000ffff00000000) >> 32, 
            (m_qwVersion & 0x00000000ffff0000) >> 16, 
             m_qwVersion & 0x000000000000ffff);
        ExitOnFailure(hr, "Failed to format version string.");

        hr = StrAllocFormatted(&sczCacheRoot, L"%lsApplications\\Cache\\%lsv%ls", sczLocalAppData, m_sczProductCode, wzVersion);
        ExitOnFailure(hr, "Failed to format cache path.");

        // now inform the payloads
        for (DWORD dwIndex = 0; dwIndex < m_cPayloads; ++dwIndex)
        {
            m_rgpPayloads[dwIndex]->SetCompletedRoot(sczCacheRoot);
        }

    LExit:
        ReleaseStr(sczCacheRoot);

        return hr;
    }




public:
    //
    // Constructor - intitialize member variables.
    //
    CBurnMsiPackage(
        __in_z LPCWSTR wzId,
        __in_z LPCWSTR wzProductCode,
        __in DWORD64 qwVersion,
        __in BOOL fPerMachine,
        __in DWORD cPayloads,
        __in_opt IBurnPayload** rgpPayloads,
        __in_opt IBurnProperties* pProperties,
        __inout HRESULT *pHR
        ) : CBurnPackageBase(wzId, qwVersion, fPerMachine, cPayloads, rgpPayloads, pHR)
    {
        HRESULT hr = *pHR;
        LPWSTR sczProductCode = NULL;
        ExitOnFailure(hr, "Failed to initialize base package.");

        if (pProperties)
        {
            m_pProperties = pProperties;
            m_pProperties->AddRef();
        }

        hr = StrAllocString(&sczProductCode, wzProductCode, 0);
        ExitOnFailure(hr, "Failed to copy product code.");

        m_sczProductCode = sczProductCode;
        sczProductCode = NULL;

    LExit:
        ReleaseStr(sczProductCode);
        *pHR = hr;
    }

    //
    // Destructor - release member variables.
    //
    ~CBurnMsiPackage()
    {
        ReleaseStr(m_sczProductCode);
    }

private:
    LPWSTR m_sczProductCode;
};


//
// BurnCreateMsiPackage - creates a new IBurnMsiPackage object.
//
HRESULT BurnCreateMsiPackage(
    __in_z LPCWSTR wzId,
    __in_z LPCWSTR wzProductCode,
    __in DWORD64 qwVersion,
    __in BOOL fPerMachine,
    __in DWORD cPayloads,
    __in_opt IBurnPayload** rgpPayloads,
    __in_opt IBurnProperties* pProperties,
    __out IBurnMsiPackage** ppMsiPackage
    )
{
    HRESULT hr = S_OK;

    CBurnMsiPackage* pMsiPackage = new CBurnMsiPackage(wzId, wzProductCode, qwVersion, fPerMachine, cPayloads, rgpPayloads, pProperties, &hr);
    ExitOnNull(pMsiPackage, hr, E_OUTOFMEMORY, "Failed to create new MSI package.");
    ExitOnFailure(hr, "Failed to initialize MSI package.");

    hr = pMsiPackage->SetPayloadCacheRoot();
    ExitOnFailure(hr, "Failed to set payload cache roots.");

    *ppMsiPackage = pMsiPackage;
    pMsiPackage = NULL;

LExit:
    ReleaseObject(pMsiPackage);
    return hr;
}
