//-------------------------------------------------------------------------------------------------
// <copyright file="BurnProperties.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"


class CBurnProperties : public CUnknownImpl<IBurnProperties>
{
public: // IBurnProperties
    //
    // GetNumericValueByName - gets numeric property value by name.
    //
    STDMETHODIMP GetNumericValueByName(
        __in_z LPCWSTR wzProperty,
        __out LONGLONG* pllValue
        )
    {
        return PropertyGetNumeric(&this->m_properties, wzProperty, pllValue);
    }


    //
    // GetStringValueByName - gets string property value by name.
    //
    STDMETHODIMP GetStringValueByName(
        __in_z LPCWSTR wzProperty,
        __out_z LPWSTR* psczValue
        )
    {
        return PropertyGetString(&this->m_properties, wzProperty, psczValue);
    }


    //
    // GetVersionValueByName - gets version property value by name.
    //
    STDMETHODIMP GetVersionValueByName(
        __in_z LPCWSTR wzProperty,
        __out DWORD64* pqwValue
        )
    {
        return PropertyGetVersion(&this->m_properties, wzProperty, pqwValue);
    }


    //
    // GetFormattedValueByName - gets formatted property value by name.
    //
    STDMETHODIMP GetFormattedValueByName(
        __in_z LPCWSTR wzProperty,
        __out_z LPWSTR* psczValue
        )
    {
        return PropertyGetFormatted(&this->m_properties, wzProperty, psczValue);
    }


    //
    // SetNumeric - adds or updates a numeric property.
    //
    STDMETHODIMP SetNumeric(
        __in_z LPCWSTR wzProperty,
        __in LONGLONG llValue
        )
    {
        return PropertySetNumeric(&this->m_properties, wzProperty, llValue);
    }


    //
    // SetString - adds or updates a string property.
    //
    STDMETHODIMP SetString(
        __in_z LPCWSTR wzProperty,
        __in_z_opt LPCWSTR wzValue
        )
    {
        return PropertySetString(&this->m_properties, wzProperty, wzValue);
    }


    //
    // SetVersion - adds or updates a version property.
    //
    STDMETHODIMP SetVersion(
        __in_z LPCWSTR wzProperty,
        __in DWORD64 qwValue
        )
    {
        return PropertySetVersion(&this->m_properties, wzProperty, qwValue);
    }


    //
    // FormatString - formats a property string.
    //
    STDMETHODIMP FormatString(
        __in_z LPCWSTR wzIn,
        __out_z LPWSTR* ppwzOut
        )
    {
        return PropertyFormatString(&this->m_properties, wzIn, ppwzOut);
    }


    //
    // EscapeString - escapes characters in a property string.
    //
    STDMETHODIMP EscapeString(
        __in_z LPCWSTR wzIn,
        __out_z LPWSTR* ppwzOut
        )
    {
        return PropertyEscapeString(wzIn, ppwzOut);
    }


    //
    // EvaluateCondition - evaluates a conditional expression.
    //
    STDMETHODIMP EvaluateCondition(
        __in_z LPCWSTR wzCondition,
        __out BOOL* pf
        )
    {
        return ConditionEvaluate(&this->m_properties, wzCondition, pf);
    }


public:
    //
    // Constructor - intitialize member variables.
    //
    CBurnProperties()
    {
        ::ZeroMemory(&this->m_properties, sizeof(BURN_PROPERTIES));
    }


    //
    // Destructor - release member variables.
    //
    ~CBurnProperties()
    {
        PropertiesUninitialize(&this->m_properties);
    }

private:
    BURN_PROPERTIES m_properties;
};


//
// BurnCreateProperties - creates a new IBurnProperties object.
//
HRESULT BurnCreateProperties(
    __out IBurnProperties **ppProperties
    )
{
    HRESULT hr = S_OK;

    CBurnProperties* pProperties = new CBurnProperties();
    ExitOnNull(pProperties, hr, E_OUTOFMEMORY, "Failed to create new properties object.");

    *ppProperties = pProperties;
    pProperties = NULL;

LExit:
    ReleaseObject(pProperties);
    return hr;
}
