//-------------------------------------------------------------------------------------------------
// <copyright file="core.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//
// <summary>
//    Setup chainer/bootstrapper core for WiX toolset.
// </summary>
//-------------------------------------------------------------------------------------------------

#pragma once


#if defined(__cplusplus)
extern "C" {
#endif


#define SETUP_INSTALL_CHAIN_ALLUSERS                0x01 // package installs to ALLUSERS
#define SETUP_INSTALL_CHAIN_PRIVILEGED              0x02 // package requires elevated privileges to install
#define SETUP_INSTALL_CHAIN_CACHE                   0x04 // cache the package before installing
#define SETUP_INSTALL_CHAIN_SHOW_UI                 0x08 // show the UI of the embedded MSI
#define SETUP_INSTALL_CHAIN_IGNORE_FAILURES         0x10 // ignore any install failures
#define SETUP_INSTALL_CHAIN_MINOR_UPGRADE_ALLOWED   0x20 // the embedded MSI can do a minor upgrade
#define SETUP_INSTALL_CHAIN_LINK                    0x30 // embedded resource is only a link to package
#define SETUP_INSTALL_CHAIN_USE_TRANSFORM           0x40 // Uses the transforms included in setup


// function pointer type declarations

typedef UINT (WINAPI *PFN_MSISETEXTERNALUIRECORD)(
    __in INSTALLUI_HANDLER_RECORD puiHandler,
    __in DWORD dwMessageFilter,
    __in_opt LPVOID pvContext,
    __out_opt PINSTALLUI_HANDLER_RECORD ppuiPrevHandler
    );
typedef UINT (WINAPI *PFN_MSIBEGINTRANSACTION)(
    __in LPCWSTR szName,
    __in DWORD dwTransactionAttributes,
    __out MSIHANDLE* phTransactionHandle,
    __out HANDLE* phChangeOfOwnerEvent
    );
typedef UINT (WINAPI *PFN_MSIENDTRANSACTION)(
    __in DWORD dwTransactionState
    );
typedef UINT (WINAPI *PFN_MSIJOINTRANSACTION)(
  __in   MSIHANDLE hTransactionID,
  __in   DWORD dwTransactionAttributes,
  __out  HANDLE *phChangeOfOwnerEvent
);


typedef BOOL (WINAPI *PFN_CREATEPROCESSW)(
    __in_opt    LPCWSTR lpApplicationName,
    __inout_opt LPWSTR lpCommandLine,
    __in_opt    LPSECURITY_ATTRIBUTES lpProcessAttributes,
    __in_opt    LPSECURITY_ATTRIBUTES lpThreadAttributes,
    __in        BOOL bInheritHandles,
    __in        DWORD dwCreationFlags,
    __in_opt    LPVOID lpEnvironment,
    __in_opt    LPCWSTR lpCurrentDirectory,
    __in        LPSTARTUPINFOW lpStartupInfo,
    __out       LPPROCESS_INFORMATION lpProcessInformation
    );

typedef BOOL (WINAPI *PFN_GETEXITCODEPROCESS)(
    __in  HANDLE hProcess,
    __out LPDWORD lpExitCode
    );

typedef UINT (WINAPI *PFN_MSIGETPRODUCTINFOW)(
    __in LPCWSTR   szProduct,
    __in LPCWSTR   szAttribute,
    __out_ecount_opt(*pcchValueBuf) LPWSTR lpValueBuf,
    __inout_opt                     LPDWORD pcchValueBuf
    );

typedef INSTALLSTATE (WINAPI *PFN_MSIQUERYFEATURESTATEW)(
    __in LPCWSTR  szProduct,
    __in LPCWSTR  szFeature
    );

typedef INSTALLUILEVEL (WINAPI *PFN_MSISETINTERNALUI)(
    __in INSTALLUILEVEL  dwUILevel,
    __inout_opt HWND  *phWnd
    );

typedef INSTALLUI_HANDLERW (WINAPI *PFN_MSISETEXTERNALUIW)(
    __in_opt INSTALLUI_HANDLERW puiHandler,
    __in DWORD              dwMessageFilter,
    __in_opt LPVOID             pvContext
    );

typedef UINT (WINAPI *PFN_MSISETEXTERNALUIRECORD)(
    __in INSTALLUI_HANDLER_RECORD puiHandler,
    __in DWORD dwMessageFilter,
    __in_opt LPVOID pvContext,
    __out_opt PINSTALLUI_HANDLER_RECORD ppuiPrevHandler
    );

typedef UINT (WINAPI *PFN_MSIENABLELOGW)(
    __in DWORD     dwLogMode,
    __in_opt LPCWSTR  szLogFile,
    __in DWORD     dwLogAttributes
    );

typedef UINT (WINAPI *PFN_MSIINSTALLPRODUCTW)(
    __in LPCWSTR      szPackagePath,
    __in_opt LPCWSTR      szCommandLine
    );

typedef UINT (WINAPI *PFN_MSICONFIGUREPRODUCTEXW)(
    __in LPCWSTR      szProduct,
    __in int          iInstallLevel,
    __in INSTALLSTATE eInstallState,
    __in_opt LPCWSTR      szCommandLine
    );


typedef LRESULT (WINAPI *PFN_SETUPUX_INITIALIZE)(
    __in SETUP_COMMAND_LINE *pSetupCommandLine,
    __in HANDLE hSetupInformation,
    __out HWND *phWnd
    );

typedef LRESULT (WINAPI *PFN_SETUPUX_UNINITIALIZE)(
    __in HANDLE hSetupInformation
    );

typedef HRESULT (WINAPI *PFN_SETUPUX_MESSAGE_LOOP)(
    __in HANDLE hSetupUXInformation
    );

typedef LRESULT (WINAPI *PFN_SETUP_MESSAGE_HANDLER)(
    __in PVOID pvContext,
    __in UINT Msg,
    __in PVOID pvData
    );


// structs

typedef struct
{
    CHAR szSource[12];
    GUID guidProductCode;
    DWORD dwVersionMajor;
    DWORD dwVersionMinor;
    DWORD dwAttributes; // any combination of the #define SETUP_INSTALL_CHAIN_
    // DWORD rgHash[8];
} SETUP_INSTALL_PACKAGE;

typedef struct
{
    CHAR szTransform[12];
    DWORD dwLocaleId;
} SETUP_INSTALL_TRANSFORM;

typedef struct
{
    BYTE dwRevision;
    BYTE cPackages;
    // Total number of transforms in package
    BYTE cTransforms;
    SETUP_INSTALL_PACKAGE rgPackages[1];
    SETUP_INSTALL_TRANSFORM rgTransforms[1];
} SETUP_INSTALL_CHAIN;

typedef struct _SETUP_SOURCE_PATH
{
    LPWSTR wzPath;
} SETUP_SOURCE_PATH;

typedef struct _SETUP_UX_RESOURCE
{
    LPWSTR wzFileName;
    LPWSTR wzDestination;
    LPWSTR wzPath;
    DWORD dwOffset;
    DWORD cbSize;
} SETUP_UX_RESOURCE;

typedef struct _SETUP_UX_RESOURCES
{
    DWORD cUXResources;
    SETUP_UX_RESOURCE rgUXResources[1];
} SETUP_UX_RESOURCES;

#ifdef DEBUG
typedef struct _SETUP_DEBUG_HOOKS
{
    PFN_CREATEPROCESSW pfnCreateProcessW;
    PFN_GETEXITCODEPROCESS pfnGetExitCodeProcess;
    PFN_MSIGETPRODUCTINFOW pfnMsiGetProductInfoW;
    PFN_MSIQUERYFEATURESTATEW pfnMsiQueryFeatureStateW;
    PFN_MSISETINTERNALUI pfnMsiSetInternalUI;
    PFN_MSISETEXTERNALUIW pfnMsiSetExternalUIW;
    PFN_MSISETEXTERNALUIRECORD pfnMsiSetExternalUIRecord;
    PFN_MSIENABLELOGW pfnMsiEnableLogW;
    PFN_MSIINSTALLPRODUCTW pfnMsiInstallProductW;
    PFN_MSICONFIGUREPRODUCTEXW pfnMsiConfigureProductExW;
} SETUP_DEBUG_HOOKS;
#endif

typedef struct _SETUP_ENGINE_INFORMATION
{
    SETUP_COMMAND_LINE *pSetupCommandLine;
    SETUP_PACKAGES *pSetupPackages;
    SETUP_UX_RESOURCES* pUXResources;

    SETUP_SOURCE_PATH* rgSourcePaths;
    DWORD cSourcePaths;

    HMODULE hMsi;
    DLLVERSIONINFO MsiVersionInfo;
    DWORD dwVersionMsi;
    PFN_MSISETEXTERNALUIRECORD pfnMsiSetExternalUIRecord;

    LPWSTR pwzPipe;
    HANDLE hPipe;
    HANDLE hThread;

    LPWSTR pwzSetupUxDll;
    LPWSTR pwzUxDirectory;

    PFN_SETUP_MESSAGE_HANDLER pfnMessageHandler;
    LPVOID pvHandlerContext;
    BOOL fEnableSourceResPrompt;

    LPWSTR pwzLogFile;
    BOOL fLogInitialized;

    BOOL fUserAbort;
    BOOL fRollback;

    // progress variables
    DWORD64 qwProgressTotal;
    DWORD64 qwProgressComplete;
    DWORD64 qwProgressStageTotal;

    // MSI progress variables
    DWORD dwMsiProgressTotal;
    DWORD dwMsiProgressStep;
    DWORD dwMsiProgressComplete;
    BOOL fMsiProgressForward;
    BOOL fMsiProgressScriptInProgress;
    BOOL fMsiProgressEnableActionData;
    BOOL fMsiProgressFinished;

    LPWSTR pwzCachePath;
} SETUP_ENGINE_INFORMATION;

typedef struct _SETUP_UX_INFORMATION
{
    SETUP_COMMAND_LINE *pSetupCommandLine;

    HMODULE hModule;
    HWND hWnd;

    PFN_SETUPUX_INITIALIZE pfnInitialize;
    PFN_SETUPUX_UNINITIALIZE pfnUninitialize;
    PFN_SETUPUX_MESSAGE_LOOP pfnMessageLoop;
} SETUP_UX_INFORMATION;

#define INVALID_LCID -1


// setup engine functions
HRESULT WIXAPI InitializeSetupEngine(
    __in SETUP_COMMAND_LINE* pSetupCommandLine,
    __out SETUP_ENGINE_INFORMATION **ppSetupUXInformation
    );
void WIXAPI UninitializeSetupEngine(
    __in_opt SETUP_ENGINE_INFORMATION *pSetupUXInformation
    );
HRESULT WIXAPI InitializeDefaultProperties(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo
    );
HRESULT WIXAPI PostWorkerMessage(
    __in SETUP_ENGINE_INFORMATION* pEngineInfo,
    __in DWORD dwMessage,
    __in LPVOID pvData,
    __in DWORD cbData
    );
void WIXAPI FreeSetupCommandLine(
    __in SETUP_COMMAND_LINE* pSetupCommandLine
    );

#ifdef DEBUG
void WIXAPI SetEngineDebugHooks(
    __in SETUP_DEBUG_HOOKS* pDebugHooks
    );
#endif
HRESULT WIXAPI SetEngineMessageHandler(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in PFN_SETUP_MESSAGE_HANDLER pfnMessageHandler,
    __in LPVOID pvContext
    );
HRESULT WIXAPI SetMsiSourceResolutionPrompt(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in BOOL fEnableSourceResPrompt
    );
HRESULT WIXAPI AddSourcePath(
    __in SETUP_ENGINE_INFORMATION *pEngineInfo,
    __in LPCWSTR wzSourcePath
    );
HRESULT WIXAPI GetLogPath(
    __in SETUP_ENGINE_INFORMATION* pEngineInfo,
    __out LPWSTR* ppwzLogPath
    );

// setup ux functions
HRESULT WIXAPI InitializeSetupUX(
    __in SETUP_ENGINE_INFORMATION* pEngineInfo,
    __in SETUP_COMMAND_LINE* pSetupCommandLine,
    __out SETUP_UX_INFORMATION** ppSetupUXInformation
    );
void WIXAPI UninitializeSetupUX(
    __in_opt SETUP_UX_INFORMATION *pSetupUXInformation
    );
HRESULT WIXAPI EnterSetupUXMessageLoop(
    __in SETUP_UX_INFORMATION* pSetupUXInformation
    );

// manifest functions
HRESULT WIXAPI LoadManifest(
    __in SETUP_ENGINE_INFORMATION* pEngineInfo
    );
void WIXAPI FreeSetupPackages(
    __in_opt SETUP_PACKAGES *pSetupPackages
    );
void WIXAPI FreeUXResources(
    __in_opt SETUP_UX_RESOURCES* pUXResources
    );
HRESULT WIXAPI AddRegistrySearch(
    __in SETUP_PACKAGES* pSetupPackages,
    __in LPCWSTR wzId,
    __in REGISTRY_SEARCH_TYPE eType,
    __in HKEY hRoot,
    __in LPWSTR wzKey,
    __in LPWSTR wzValue,
    __in LPWSTR wzProperty
    );
HRESULT WIXAPI SetPackageProperty(
    __in SETUP_PACKAGE* pPackage,
    __in LPCWSTR wzProperty,
    __in_opt LPCWSTR wzRequestedValue,
    __in_opt LPCWSTR wzExpectedValue
    );

#if defined(__cplusplus)
}
#endif
