//-------------------------------------------------------------------------------------------------
// <copyright file="IInstallationAbstractionLayer.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#pragma once

#define IDERROR -1
#define IDNOACTION 0

typedef enum INSTALL_TRANSACTION
{
    INSTALL_TRANSACTION_ROLLBACK = 0,
    INSTALL_TRANSACTION_COMMIT
};

DECLARE_INTERFACE_IID_(IInstallationAbstractionLayer, IUnknown, "d3897d74-67a0-43a8-9d93-a49007a33d09")
{
    STDMETHOD(CreateProcess)(
        __in LPCWSTR lpApplicationName,
        __inout LPWSTR lpCommandLine,
        __in_opt LPSECURITY_ATTRIBUTES lpProcessAttributes,
        __in_opt LPSECURITY_ATTRIBUTES lpThreadAttributes,
        __in BOOL bInheritHandles,
        __in DWORD dwCreationFlags,
        __in_opt LPVOID lpEnvironment,
        __in_opt LPCWSTR lpCurrentDirectory,
        __in LPSTARTUPINFOW lpStartupInfo,
        __out LPPROCESS_INFORMATION lpProcessInformation
        ) PURE;

    STDMETHOD(GetExitCodeProcess)(
        __in HANDLE hProcess,
        __out LPDWORD lpExitCode
        ) PURE;

    STDMETHOD(IsElevated)() PURE;

    STDMETHOD(MsiGetProductInfo)(
        __in LPCWSTR szProduct,
        __in LPCWSTR szProperty,
        __out LPWSTR* psczValue
        ) PURE;

    STDMETHOD_(INSTALLSTATE, MsiQueryFeatureState)(
        __in LPCWSTR szProduct,
        __in LPCWSTR szFeature
        ) PURE;

    STDMETHOD_(INSTALLUILEVEL, MsiSetInternalUI)(
        __in INSTALLUILEVEL dwUILevel,
        __inout HWND* phWnd
        ) PURE;

    STDMETHOD(MsiSetExternalUI)(
        __in INSTALLUI_HANDLERW puiHandler,
        __in DWORD dwMessageFilter,
        __in LPVOID pvContext,
        __out_opt INSTALLUI_HANDLERW* ppuiPrevHandler
        ) PURE;

    STDMETHOD(MsiSetExternalUIRecord)(
        __in INSTALLUI_HANDLER_RECORD puiHandler,
        __in DWORD dwMessageFilter,
        __in LPVOID pvContext,
        __out_opt PINSTALLUI_HANDLER_RECORD ppuiPrevHandler
        ) PURE;

    STDMETHOD(MsiEnableLog)(
        __in DWORD dwLogMode,
        __in LPCWSTR szLogFile,
        __in DWORD dwLogAttributes
        ) PURE;

    STDMETHOD(MsiBeginTransaction)(
        __in LPCWSTR szName,
        __in DWORD dwTransactionAttributes,
        __out MSIHANDLE* phTransactionHandle,
        __out HANDLE* phChangeOfOwnerEvent
        ) PURE;

    STDMETHOD(MsiJoinTransaction)(
        __in   MSIHANDLE hTransactionID,
        __in   DWORD dwTransactionAttributes,
        __out  HANDLE *phChangeOfOwnerEvent
        ) PURE;

    STDMETHOD(MsiEndTransaction)(
        __in DWORD dwTransactionState
        ) PURE;

    STDMETHOD(MsiInstallProduct)(
        __in LPCWSTR szPackagePath,
        __in LPCWSTR szCommandLine
        ) PURE;

    STDMETHOD(MsiConfigureProductEx)(
        __in LPCWSTR szProduct,
        __in int iInstallLevel,
        __in INSTALLSTATE eInstallState,
        __in LPCWSTR szCommandLine
        ) PURE;

    STDMETHOD(ScheduleRestart)() PURE;

    STDMETHOD(ShellExecute)(
        __in SHELLEXECUTEINFOW* pInfo
        ) PURE;
};

HRESULT BurnCreateInstallationAbstractionLayer(
    __out IInstallationAbstractionLayer **ppInstallationAbstractionLayer
    );
