//-------------------------------------------------------------------------------------------------
// <copyright file="stub.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//
// <summary>
// Setup chainer/bootstrapper executable for WiX toolset.
// </summary>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

#define HRESULT_FROM_RPC(x)      ((HRESULT) ((x) | FACILITY_RPC))


int WINAPI WinMain(
    __in HINSTANCE hInstance,
    __in HINSTANCE hPrevInstance,
    __in LPSTR lpCmdLine,
    __in int nCmdShow
    )
{
    HRESULT hr = S_OK;
    LPWSTR sczCommandLine = NULL;
    BURN_MODE mode = BURN_MODE_NORMAL;
    IBurnModel* pModel = NULL;
    IBurnView* pView = NULL;
    IInstallationAbstractionLayer* pInstallationAbstractionLayer = NULL;
    IBurnExecuteEngine* pExeEngine = NULL;
    IBurnExecuteEngine* pMsiEngine = NULL;
    IBurnCacheEngine* pExtractEngine = NULL;
    IBurnCacheEngine* pDownloadEngine = NULL;
    IBurnController* pController = NULL;

    hr = StrAllocStringAnsi(&sczCommandLine, lpCmdLine, 0, CP_ACP);
    ExitOnFailure(hr, "Failed to allocate copy of command line.");

    hr = BurnCreateModel(sczCommandLine, &pModel);
    ExitOnFailure(hr, "Failed to create model.");

    mode = pModel->Mode();
    switch (mode)
    {
    case BURN_MODE_ELEVATED:
        hr = BurnCreateViewElevated(&pView);
        ExitOnFailure(hr, "Failed to create view.");
        break;
    case BURN_MODE_NORMAL:
        hr = BurnCreateView(nCmdShow, &pView);
        ExitOnFailure(hr, "Failed to create view.");
        break;
    }

    hr = BurnCreateInstallationAbstractionLayer(&pInstallationAbstractionLayer);
    ExitOnFailure(hr, "Failed to create installation abstraction layer.");

    hr = BurnCreateExeEngine(pInstallationAbstractionLayer, &pExeEngine);
    ExitOnFailure(hr, "Failed to create executable execution engine.");

    hr = BurnCreateMsiEngine(pInstallationAbstractionLayer, &pMsiEngine);
    ExitOnFailure(hr, "Failed to create Windows Installer execution engine.");

    hr = BurnCreateExtractEngine(&pExtractEngine);
    ExitOnFailure(hr, "Failed to create extracting cache engine.");

    hr = BurnCreateDownloadEngine(&pDownloadEngine);
    ExitOnFailure(hr, "Failed to create downloading cache engine.");

    hr = BurnCreateController(pModel, pView, pExeEngine, pMsiEngine, pExtractEngine, pDownloadEngine, pInstallationAbstractionLayer, &pController);
    ExitOnFailure(hr, "Failed to create controller.");

    hr = pModel->Load(NULL);
    ExitOnFailure(hr, "Failed to load model.");

    hr = pController->Run();
    ExitOnFailure(hr, "Controller failed during run.");

LExit:
    ReleaseObject(pController);
    ReleaseObject(pExtractEngine);
    ReleaseObject(pMsiEngine);
    ReleaseObject(pExeEngine);
    ReleaseObject(pInstallationAbstractionLayer);
    ReleaseObject(pView);
    ReleaseObject(pModel);
    ReleaseStr(sczCommandLine);

    return SCODE_CODE(hr);
}
