//-------------------------------------------------------------------------------------------------
// <copyright file="BurnExePackageTestShim.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
//
// Constructor
//
BurnExePackageTestShim::BurnExePackageTestShim(String^ id, String^ productCode, Version^ version, bool perMachine, String^ detectCondition, String^ installArguments, String^ repairArguments, String^ uninstallArguments)
{
    pin_ptr<const WCHAR> wzId = PtrToStringChars(id);
    pin_ptr<const WCHAR> wzProductCode = PtrToStringChars(productCode);
    DWORD64 qwVersion = (static_cast<DWORD64>(version->Major) << 48) |
                        (static_cast<DWORD64>(version->Minor) << 32) |
                        (static_cast<DWORD64>(version->Build) < 16) |
                        static_cast<DWORD64>(version->Revision);
    pin_ptr<const WCHAR> wzDetectCondition = PtrToStringChars(detectCondition);
    pin_ptr<const WCHAR> wzInstallArgs = PtrToStringChars(installArguments);
    pin_ptr<const WCHAR> wzRepairArgs = PtrToStringChars(repairArguments);
    pin_ptr<const WCHAR> wzUninstallArgs = PtrToStringChars(uninstallArguments);

    IBurnExePackage* x = NULL;
    HRESULT hr = BurnCreateExePackage(wzId, wzProductCode, qwVersion, perMachine ? TRUE : FALSE, wzDetectCondition, wzInstallArgs, wzRepairArgs, wzUninstallArgs, 0, NULL, 0, NULL, NULL, &x);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to create MSI package.");
    }

    this->m_pExePackage = x;

    this->m_pExePackage->AddRef();
    this->m_pPackage = this->m_pExePackage;
}


BurnExePackageTestShim::BurnExePackageTestShim(IBurnExePackage* pExePackage)
{
    pExePackage->AddRef();
    this->m_pExePackage = pExePackage;

    this->m_pExePackage->AddRef();
    this->m_pPackage = this->m_pExePackage;
}


//
// Destructor
//
BurnExePackageTestShim::~BurnExePackageTestShim()
{
    this->Destroy();
}


//
// Finalizer
//
BurnExePackageTestShim::!BurnExePackageTestShim()
{
    this->Destroy();
}


//
// Destroy - cleans up the object (called by destructor and finalizer).
//
void BurnExePackageTestShim::Destroy()
{
    ReleaseNullObject(this->m_pExePackage);
}
}
}
}
}
}
