//-------------------------------------------------------------------------------------------------
// <copyright file="BurnModelTestShim.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
//
// Command - calls IBurnModel::GetCommand().
//
BurnCommand^ BurnModelTestShim::Command::get()
{
    interior_ptr<IBurnModel*> ppModel = &(this->m_pModel);
    IBurnModel* pThisModel = *ppModel;

    BURN_COMMAND command;
    HRESULT hr = pThisModel->GetCommand(&command);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to get command.");
    }

    BurnCommand^ ret = gcnew BurnCommand();
    ret->action = (BurnAction)command.action;
    ret->display = (BurnDisplay)command.display;
    ret->restart = (BurnRestart)command.restart;

    return ret;
}


//
// ElevatedPipeName - call IBurnModel::GetElevatedInfo().
//
String^ BurnModelTestShim::ElevatedPipeName::get()
{
    interior_ptr<IBurnModel*> ppModel = &(this->m_pModel);
    IBurnModel* pThisModel = *ppModel;

    LPWSTR sczPipeName = NULL;
    LPWSTR sczToken = NULL;
    try
    {
        HRESULT hr = pThisModel->GetElevatedInfo(&sczPipeName, &sczToken);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get model elevated info.");
        }

        return gcnew String(sczPipeName);
    }
    finally
    {
        ReleaseStr(sczPipeName);
        ReleaseStr(sczToken);
    }
}


//
// ElevatedToken - call IBurnModel::GetElevatedInfo().
//
String^ BurnModelTestShim::ElevatedToken::get()
{
    interior_ptr<IBurnModel*> ppModel = &(this->m_pModel);
    IBurnModel* pThisModel = *ppModel;

    LPWSTR sczPipeName = NULL;
    LPWSTR sczToken = NULL;
    try
    {
        HRESULT hr = pThisModel->GetElevatedInfo(&sczPipeName, &sczToken);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get model elevated info.");
        }

        return gcnew String(sczToken);
    }
    finally
    {
        ReleaseStr(sczPipeName);
        ReleaseStr(sczToken);
    }
}

//
// LogPath - call IBurnModel::GetLogPath().
//
String^ BurnModelTestShim::LogPath::get()
{
    interior_ptr<IBurnModel*> ppModel = &(this->m_pModel);
    IBurnModel* pThisModel = *ppModel;

    LPWSTR scz = NULL;
    try
    {
        HRESULT hr = pThisModel->GetLogPath(&scz, NULL);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get model log path.");
        }

        return gcnew String(scz);
    }
    finally
    {
        ReleaseStr(scz);
    }
}


//
// Mode - calls IBurnModel::Mode().
//
BurnMode^ BurnModelTestShim::Mode::get()
{
    interior_ptr<IBurnModel*> ppModel = &(this->m_pModel);
    IBurnModel* pThisModel = *ppModel;

    BurnMode^ ret = (BurnMode)pThisModel->Mode();
    return ret;
}


//
// PackageCount - calls IBurnModel::PackageCount().
//
DWORD BurnModelTestShim::PackageCount::get()
{
    interior_ptr<IBurnModel*> ppModel = &(this->m_pModel);
    IBurnModel* pThisModel = *ppModel;

    DWORD cPackages = pThisModel->PackageCount();
    return cPackages;
}


//
// Load - call IBurnModel::Load().
//
void BurnModelTestShim::Load(String^ bundleFileName)
{
    pin_ptr<const WCHAR> wzBundle = PtrToStringChars(bundleFileName);
    interior_ptr<IBurnModel*> ppModel = &(this->m_pModel);
    IBurnModel* pThisModel = *ppModel;

    HRESULT hr = pThisModel->Load(wzBundle);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to load model.");
    }
}


//
// GetPackage - call IBurnModel::GetPackage().
//
BurnPackageTestShim^ BurnModelTestShim::GetPackage(int index)
{
    interior_ptr<IBurnModel*> ppModel = &(this->m_pModel);
    IBurnModel* pThisModel = *ppModel;

    IBurnPackage* pPackage = NULL;

    try
    {
        HRESULT hr = pThisModel->GetPackage(index, &pPackage);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get package.");
        }

        return BurnPackageTestShim::Create(pPackage);
    }
    finally
    {
        ReleaseObject(pPackage);
    }
}


void BurnModelTestShim::AddPackage(BurnPackageTestShim^ package)
{
    interior_ptr<IBurnModel*> ppModel = &(this->m_pModel);
    IBurnModel* pThisModel = *ppModel;

    interior_ptr<IBurnPackage*> ppPackage = &(package->m_pPackage);

    HRESULT hr;
    IBurnModelTestable* pTestableModel = NULL;
    try
    {
        hr = pThisModel->QueryInterface(IID_PPV_ARGS(&pTestableModel));
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to query for testable model.");
        }

        hr = pTestableModel->AddPackageForTestingPurposes(*ppPackage);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to add package to testable model.");
        }
    }
    finally
    {
        ReleaseObject(pTestableModel);
    }
}

//
// Constructor
//
BurnModelTestShim::BurnModelTestShim(String^ commandLine)
{
    pin_ptr<const WCHAR> wzCommandLine = PtrToStringChars(commandLine);

    IBurnModel* x = NULL;
    HRESULT hr = BurnCreateModel(wzCommandLine, &x);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to create model.");
    }

    this->m_pModel = x;
}


//
// Constructor - this is only for early test purposes, remove when find a better way to test.
//
//BurnModelTestShim::BurnModelTestShim(BurnMsiPackageTestShim^ msiPackage)
//{
//    interior_ptr<IBurnMsiPackage*> ppMsiPackage = &(msiPackage->m_pMsiPackage);
//
//    IBurnModel* x = NULL;
//    HRESULT hr = BurnCreateModel(*ppMsiPackage, &x);
//    if (FAILED(hr))
//    {
//        throw gcnew BurnTestShimException(hr, "Failed to create model.");
//    }
//
//    this->m_pModel = x;
//}


//
// Constructor
//
BurnModelTestShim::BurnModelTestShim(IBurnModel* pModel)
{
    pModel->AddRef();
    this->m_pModel = pModel;
}


//
// Destructor
//
BurnModelTestShim::~BurnModelTestShim()
{
    this->Destroy();
}


//
// Finalizer
//
BurnModelTestShim::!BurnModelTestShim()
{
    this->Destroy();
}


//
// Destroy - cleans up the object (called by destructor and finalizer).
//
void BurnModelTestShim::Destroy()
{
    ReleaseNullObject(this->m_pModel);
}
}
}
}
}
}
