//-------------------------------------------------------------------------------------------------
// <copyright file="BurnModelTestShim.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#pragma once

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
    public enum class BurnAction
    {
        Unknown,
        Help,
        Install,
        Uninstall,
    };

    public enum class BurnDisplay
    {
        Unknown,
        None,
        Passive,
        Full,
    };

    public enum class BurnRestart
    {
        Unknown,
        Never,
        Prompt,
        Automatic,
        Always,
    };

    public ref struct BurnCommand
    {
        BurnAction action;
        BurnDisplay display;
        BurnRestart restart;
    };

    public enum class BurnMode
    {
        Normal,
        Elevated,
        Embedded,
    };

    public ref class BurnModelTestShim
    {
    public:
        property BurnCommand^ Command { BurnCommand^ get(); };

        property String^ ElevatedPipeName { String^ get(); };

        property String^ ElevatedToken { String^ get(); };

        property String^ LogPath { String^ get(); };

        property BurnMode^ Mode { BurnMode^ get(); };

        property DWORD PackageCount { DWORD get(); };

        void Load(String^ bundleFileName);

        BurnPackageTestShim^ GetPackage(int index);

        void AddPackage(BurnPackageTestShim^ package);

    public:
        BurnModelTestShim(String^ commandLine);

        ~BurnModelTestShim();

    internal:
        BurnModelTestShim(IBurnModel* pModel);

    protected:
        !BurnModelTestShim();

    private:
        void Destroy();

    internal:
        IBurnModel* m_pModel;
    };
}
}
}
}
}
