//-------------------------------------------------------------------------------------------------
// <copyright file="BurnMsiPackageTestShim.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
//
// ProductCode - call IBurnMsiPackage::GetProductCode().
//
String^ BurnMsiPackageTestShim::ProductCode::get()
{
    interior_ptr<IBurnMsiPackage*> ppMsiPackage = &(this->m_pMsiPackage);
    IBurnMsiPackage* pThisMsiPackage = *ppMsiPackage;

    LPWSTR sczProductCode = NULL;
    try
    {
        HRESULT hr = pThisMsiPackage->GetProductCode(&sczProductCode);
        if (FAILED(hr))
        {
            throw gcnew BurnTestShimException(hr, "Failed to get product code.");
        }

        return gcnew String(sczProductCode);
    }
    finally
    {
        ReleaseStr(sczProductCode);
    }
}


//
// Constructor
//
BurnMsiPackageTestShim::BurnMsiPackageTestShim(String^ id, String^ productCode, Version^ version, bool perMachine)
{
    pin_ptr<const WCHAR> wzId = PtrToStringChars(id);
    pin_ptr<const WCHAR> wzProductCode = PtrToStringChars(productCode);
    DWORD64 qwVersion = (static_cast<DWORD64>(version->Major) << 48) |
                        (static_cast<DWORD64>(version->Minor) << 32) |
                        (static_cast<DWORD64>(version->Build) < 16) |
                        static_cast<DWORD64>(version->Revision);

    IBurnMsiPackage* x = NULL;
    HRESULT hr = BurnCreateMsiPackage(wzId, wzProductCode, qwVersion, perMachine ? TRUE : FALSE, 0, NULL, NULL, &x);
    if (FAILED(hr))
    {
        throw gcnew BurnTestShimException(hr, "Failed to create MSI package.");
    }

    this->m_pMsiPackage = x;

    this->m_pMsiPackage->AddRef();
    this->m_pPackage = this->m_pMsiPackage;
}


BurnMsiPackageTestShim::BurnMsiPackageTestShim(IBurnMsiPackage* pMsiPackage)
{
    pMsiPackage->AddRef();
    this->m_pMsiPackage = pMsiPackage;

    this->m_pMsiPackage->AddRef();
    this->m_pPackage = this->m_pMsiPackage;
}


//
// Destructor
//
BurnMsiPackageTestShim::~BurnMsiPackageTestShim()
{
    this->Destroy();
}


//
// Finalizer
//
BurnMsiPackageTestShim::!BurnMsiPackageTestShim()
{
    this->Destroy();
}


//
// Destroy - cleans up the object (called by destructor and finalizer).
//
void BurnMsiPackageTestShim::Destroy()
{
    ReleaseNullObject(this->m_pMsiPackage);
}
}
}
}
}
}
