//-------------------------------------------------------------------------------------------------
// <copyright file="BurnViewMockBase.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

using namespace System::Runtime::InteropServices;


namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
//
// InternalInit - called by controller with native objects to initialize.
//
HRESULT BurnViewMockBase::InternalInit(IBurnModel* pModel, IBurnController* pController)
{
    BurnModelTestShim^ model = gcnew BurnModelTestShim(pModel);
    BurnControllerTestShim^ controller = gcnew BurnControllerTestShim(pController);

    return this->Initialize(model, controller);
}


//
// InternalOnPrepareBegin - converts native types to managed types.
//
BOOL BurnViewMockBase::InternalOnPrepareBegin(DWORD cPackages)
{
    return this->OnPrepareBegin(cPackages) ? TRUE : FALSE;
}


//
// InternalOnPreparePackageBegin - converts native package to managed package test shim.
//
BOOL BurnViewMockBase::InternalOnPreparePackageBegin(IBurnPackage* pPackage)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);

    return this->OnPreparePackageBegin(package) ? TRUE : FALSE;
}


//
// InternalOnPreparePackageBegin - converts native package to managed package test shim.
//
void BurnViewMockBase::InternalOnPreparePackageComplete(IBurnPackage* pPackage, HRESULT hrStatus)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);

    this->OnPreparePackageComplete(package, hrStatus);
}


//
// InternalOnCacheBegin - converts native types to managed types.
//
BOOL BurnViewMockBase::InternalOnCacheBegin(DWORD cCachingPackages, DWORD cTotalPayloads, DWORD64 dw64TotalCacheSize)
{
    return this->OnCacheBegin(cCachingPackages, cTotalPayloads, dw64TotalCacheSize) ? TRUE : FALSE;
}


//
// InternalOnCachePackageBegin - converts native package to managed package test shim.
//
BOOL BurnViewMockBase::InternalOnCachePackageBegin(IBurnPackage* pPackage, DWORD cPayloads, DWORD64 dw64PackageCacheSize)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);

    return this->OnCachePackageBegin(package, cPayloads, dw64PackageCacheSize) ? TRUE : FALSE;
}


//
// InternalOnCachePayloadBegin - converts native package to managed package test shim.
//
BOOL BurnViewMockBase::InternalOnCachePayloadBegin(IBurnPackage* pPackage, IBurnPayload* pPayload, SOURCE_TYPE sourceType, LPCWSTR wzSourcePath)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);
    BurnPayloadTestShim^ payload = gcnew BurnPayloadTestShim(pPayload);
    String^ sourcePath = gcnew String(wzSourcePath);

    return this->OnCachePackagePayloadBegin(package, payload, (PayloadSourceType)sourceType, sourcePath) ? TRUE : FALSE;
}


//
// InternalOnCachePayloadProgress - converts native package to managed package test shim.
//
BOOL BurnViewMockBase::InternalOnCachePayloadProgress(IBurnPackage* pPackage, IBurnPayload* pPayload, DWORD64 dw64PayloadCached, DWORD64 dw64PayloadSize)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);
    BurnPayloadTestShim^ payload = gcnew BurnPayloadTestShim(pPayload);

    return this->OnCachePackagePayloadProgress(package, payload, dw64PayloadCached, dw64PayloadSize) ? TRUE : FALSE;
}


//
// InternalOnCachePayloadComplete - converts native package to managed package test shim.
//
void BurnViewMockBase::InternalOnCachePayloadComplete(IBurnPackage* pPackage, IBurnPayload* pPayload, HRESULT hrStatus)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);
    BurnPayloadTestShim^ payload = gcnew BurnPayloadTestShim(pPayload);

    this->OnCachePackagePayloadComplete(package, payload, hrStatus);
}


//
// InternalOnCachePackageComplete - converts native package to managed package test shim.
//
void BurnViewMockBase::InternalOnCachePackageComplete(IBurnPackage* pPackage, HRESULT hrStatus)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);

    this->OnCachePackageComplete(package, hrStatus);
}


//
// InternalOnExecuteBegin - converts native types to managed types.
//
BOOL BurnViewMockBase::InternalOnExecuteBegin(DWORD cExecutingPackages)
{
    return this->OnExecuteBegin(cExecutingPackages) ? TRUE : FALSE;
}


//
// InternalOnExecutePackageBegin - converts native package to managed package test shim.
//
BOOL BurnViewMockBase::InternalOnExecutePackageBegin(IBurnPackage* pPackage, BOOL fExecute)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);

    return this->OnExecutePackageBegin(package, fExecute ? true : false) ? TRUE : FALSE;
}


//
// InternalOnExecutePackageComplete - converts native package to managed package test shim.
//
void BurnViewMockBase::InternalOnExecutePackageComplete(IBurnPackage* pPackage, HRESULT hrStatus)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);

    this->OnExecutePackageComplete(package, hrStatus);
}


//
// InternalOnError - converts native package to managed package test shim.
//
INT BurnViewMockBase::InternalOnError(IBurnPackage* pPackage, DWORD dwCode, LPCWSTR wzMessage, UINT uiFlags)
{
    BurnPackageTestShim^ package = BurnPackageTestShim::Create(pPackage);
    String^ message = gcnew String(wzMessage);
    MessageBoxButtons buttons = (MessageBoxButtons)(uiFlags & 0xF);
    MessageBoxIcon icon = (MessageBoxIcon)(uiFlags & 0xF >> 1);

    DialogResult result = this->OnError(dwCode, message, buttons, icon);
    return static_cast<INT>(result);
}


//
// Initialize - base do nothing call.
//
HRESULT BurnViewMockBase::Initialize(
    BurnModelTestShim^ model,
    BurnControllerTestShim^ controller
    )
{
    return S_OK;
}


//
// Run - base do nothing call.
//
HRESULT BurnViewMockBase::Run()
{
    return S_OK;
}


//
// Uninitialize - base do nothing call.
//
void BurnViewMockBase::Uninitialize()
{
}


bool BurnViewMockBase::OnPrepareBegin(DWORD packageCount)
{
    return true;
}


//
// OnPreparePackageBegin - base do nothing call.
//
bool BurnViewMockBase::OnPreparePackageBegin(BurnPackageTestShim^ package)
{
    return true;
}


//
// OnPreparePackageComplete - base do nothing call.
//
void BurnViewMockBase::OnPreparePackageComplete(BurnPackageTestShim^ package, HRESULT status)
{
}


//
// OnPrepareComplete - base do nothing call.
//
void BurnViewMockBase::OnPrepareComplete(HRESULT status)
{
}


//
// OnCacheBegin - base do nothing call.
//
bool BurnViewMockBase::OnCacheBegin(DWORD cCachingPackages, DWORD cTotalPayloads, DWORD64 dw64TotalCacheSize)
{
    return true;
}


//
// OnCachePackageBegin - base do nothing call.
//
bool BurnViewMockBase::OnCachePackageBegin(BurnPackageTestShim^ package, DWORD cPayloads, DWORD64 dw64PackageCacheSize)
{
    return true;
}


//
// OnCachePayloadBegin - base do nothing call.
//
bool BurnViewMockBase::OnCachePackagePayloadBegin(BurnPackageTestShim^ package, BurnPayloadTestShim^ payload, PayloadSourceType sourceType, String^ wzSourcePath)
{
    return true;
}


//
// OnCachePayloadProgress - base do nothing call.
//
bool BurnViewMockBase::OnCachePackagePayloadProgress(BurnPackageTestShim^ package, BurnPayloadTestShim^ payload, DWORD64 dw64PayloadCached, DWORD64 dw64PayloadSize)
{
    return true;
}


//
// OnCachePayloadComplete - base do nothing call.
//
void BurnViewMockBase::OnCachePackagePayloadComplete(BurnPackageTestShim^ package, BurnPayloadTestShim^ payload, HRESULT hrStatus)
{
}


//
// OnCachePackageComplete - base do nothing call.
//
void BurnViewMockBase::OnCachePackageComplete(BurnPackageTestShim^ package, HRESULT hrStatus)
{
}


//
// OnCacheComplete - base do nothing call.
//
void BurnViewMockBase::OnCacheComplete(HRESULT status)
{
}


bool BurnViewMockBase::OnExecuteBegin(DWORD executingCount)
{
    return true;
}


//
// OnExecutePackageBegin - base do nothing call.
//
bool BurnViewMockBase::OnExecutePackageBegin(BurnPackageTestShim^ package, bool execute)
{
    return true;
}


//
// OnProgress - base do nothing call.
//
bool BurnViewMockBase::OnProgress(DWORD percentage, DWORD overallPercentage)
{
    return true;
}


//
// OnExecutePackageComplete - base do nothing call.
//
void BurnViewMockBase::OnExecutePackageComplete(BurnPackageTestShim^ package, HRESULT status)
{
}



//
// OnExecuteComplete - base do nothing call.
//
void BurnViewMockBase::OnExecuteComplete(HRESULT status)
{
}


//
// OnError - base always cancels.
//
DialogResult BurnViewMockBase::OnError(DWORD dwCode, String^ message, MessageBoxButtons buttons, MessageBoxIcon icon)
{
    return DialogResult::Cancel;
}


//
// Constructor
//
BurnViewMockBase::BurnViewMockBase()
{
    this->m_pMockProxy = new CBurnViewMockProxy();

    this->init = gcnew InitDelegate(this, &BurnViewMockBase::InternalInit);
    this->m_pMockProxy->m_pfnInternalInit = static_cast<LPINIT>(Marshal::GetFunctionPointerForDelegate(this->init).ToPointer());

    this->run = gcnew NoArgumentsDelegate(this, &BurnViewMockBase::Run);
    this->m_pMockProxy->m_pfnRun = static_cast<LPNOARGUMENTS>(Marshal::GetFunctionPointerForDelegate(this->run).ToPointer());

    this->uninit = gcnew NoArgumentsNoReturnDelegate(this, &BurnViewMockBase::Uninitialize);
    this->m_pMockProxy->m_pfnUninitialize = static_cast<LPNOARGUMENTSNORETURN>(Marshal::GetFunctionPointerForDelegate(this->uninit).ToPointer());

    // Prepare delegates.
    this->onPrepareBegin = gcnew BeginDelegate(this, &BurnViewMockBase::InternalOnPrepareBegin);
    this->m_pMockProxy->m_pfnOnPrepareBegin = static_cast<LPBEGIN>(Marshal::GetFunctionPointerForDelegate(this->onPrepareBegin).ToPointer());

    this->onPreparePackageBegin = gcnew PackageBeginDelegate(this, &BurnViewMockBase::InternalOnPreparePackageBegin);
    this->m_pMockProxy->m_pfnOnPreparePackageBegin = static_cast<LPPACKAGEBEGIN>(Marshal::GetFunctionPointerForDelegate(this->onPreparePackageBegin).ToPointer());

    this->onPreparePackageComplete = gcnew PackageCompleteDelegate(this, &BurnViewMockBase::InternalOnPreparePackageComplete);
    this->m_pMockProxy->m_pfnOnPreparePackageComplete = static_cast<LPPACKAGECOMPLETE>(Marshal::GetFunctionPointerForDelegate(this->onPreparePackageComplete).ToPointer());

    this->onPrepareComplete = gcnew CompleteDelegate(this, &BurnViewMockBase::OnPrepareComplete);
    this->m_pMockProxy->m_pfnOnPrepareComplete = static_cast<LPCOMPLETE>(Marshal::GetFunctionPointerForDelegate(this->onPrepareComplete).ToPointer());

    // Cache delegates.
    this->onCacheBegin = gcnew CacheBeginDelegate(this, &BurnViewMockBase::InternalOnCacheBegin);
    this->m_pMockProxy->m_pfnOnCacheBegin = static_cast<LPCACHEBEGIN>(Marshal::GetFunctionPointerForDelegate(this->onCacheBegin).ToPointer());

    this->onCacheBegin = gcnew CacheBeginDelegate(this, &BurnViewMockBase::InternalOnCacheBegin);
    this->m_pMockProxy->m_pfnOnCacheBegin = static_cast<LPCACHEBEGIN>(Marshal::GetFunctionPointerForDelegate(this->onCacheBegin).ToPointer());

    this->onCachePackageBegin = gcnew PackageBeginCacheDelegate(this, &BurnViewMockBase::InternalOnCachePackageBegin);
    this->m_pMockProxy->m_pfnOnCachePackageBegin = static_cast<LPPACKAGEBEGINCACHE>(Marshal::GetFunctionPointerForDelegate(this->onCachePackageBegin).ToPointer());

    this->onCachePackagePayloadBegin = gcnew PayloadBeginCacheDelegate(this, &BurnViewMockBase::InternalOnCachePayloadBegin);
    this->m_pMockProxy->m_pfnOnCachePackagePayloadBegin = static_cast<LPPAYLOADBEGINCACHE>(Marshal::GetFunctionPointerForDelegate(this->onCachePackagePayloadBegin).ToPointer());

    this->onCacheProgress = gcnew PayloadProgressDelegate(this, &BurnViewMockBase::InternalOnCachePayloadProgress);
    this->m_pMockProxy->m_pfnOnCacheProgress = static_cast<LPPAYLOADPROGRESS>(Marshal::GetFunctionPointerForDelegate(this->onCacheProgress).ToPointer());

    this->onCachePackagePayloadComplete = gcnew PayloadCompleteDelegate(this, &BurnViewMockBase::InternalOnCachePayloadComplete);
    this->m_pMockProxy->m_pfnOnCachePackagePayloadComplete = static_cast<LPPAYLOADCOMPLETE>(Marshal::GetFunctionPointerForDelegate(this->onCachePackagePayloadComplete).ToPointer());

    this->onCachePackageComplete = gcnew PackageCompleteDelegate(this, &BurnViewMockBase::InternalOnCachePackageComplete);
    this->m_pMockProxy->m_pfnOnCachePackageComplete = static_cast<LPPACKAGECOMPLETE>(Marshal::GetFunctionPointerForDelegate(this->onCachePackageComplete).ToPointer());

    this->onCacheComplete = gcnew CompleteDelegate(this, &BurnViewMockBase::OnCacheComplete);
    this->m_pMockProxy->m_pfnOnCacheComplete = static_cast<LPCOMPLETE>(Marshal::GetFunctionPointerForDelegate(this->onCacheComplete).ToPointer());

    // Execute delegates.
    this->onExecuteBegin = gcnew BeginDelegate(this, &BurnViewMockBase::InternalOnExecuteBegin);
    this->m_pMockProxy->m_pfnOnExecuteBegin = static_cast<LPBEGIN>(Marshal::GetFunctionPointerForDelegate(this->onExecuteBegin).ToPointer());

    this->onExecutePackageBegin = gcnew PackageBeginExecuteDelegate(this, &BurnViewMockBase::InternalOnExecutePackageBegin);
    this->m_pMockProxy->m_pfnOnExecutePackageBegin = static_cast<LPPACKAGEBEGINEXECUTE>(Marshal::GetFunctionPointerForDelegate(this->onExecutePackageBegin).ToPointer());

    this->onExecuteProgress = gcnew ProgressDelegate(this, &BurnViewMockBase::OnProgress);
    this->m_pMockProxy->m_pfnProgress = static_cast<LPPROGRESS>(Marshal::GetFunctionPointerForDelegate(this->onExecuteProgress).ToPointer());

    this->onExecutePackageComplete = gcnew PackageCompleteDelegate(this, &BurnViewMockBase::InternalOnExecutePackageComplete);
    this->m_pMockProxy->m_pfnOnExecutePackageComplete = static_cast<LPPACKAGECOMPLETE>(Marshal::GetFunctionPointerForDelegate(this->onExecutePackageComplete).ToPointer());

    this->onExecuteComplete = gcnew CompleteDelegate(this, &BurnViewMockBase::OnExecuteComplete);
    this->m_pMockProxy->m_pfnOnExecuteComplete = static_cast<LPCOMPLETE>(Marshal::GetFunctionPointerForDelegate(this->onExecuteComplete).ToPointer());

    this->onError = gcnew ErrorDelegate(this, &BurnViewMockBase::InternalOnError);
    this->m_pMockProxy->m_pfnOnError = static_cast<LPERROR>(Marshal::GetFunctionPointerForDelegate(this->onError).ToPointer());
}


//
// Destructor
//
BurnViewMockBase::~BurnViewMockBase()
{
    this->Destroy();
}


//
// Finalizer
//
BurnViewMockBase::!BurnViewMockBase()
{
    this->Destroy();
}


//
// Destroy - cleans up the object (called by destructor and finalizer).
//
void BurnViewMockBase::Destroy()
{
    ReleaseNullObject(this->m_pMockProxy);
}
}
}
}
}
}
