//-------------------------------------------------------------------------------------------------
// <copyright file="BurnViewMockBase.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#pragma once

using namespace System::Windows::Forms;

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
    typedef HRESULT (CALLBACK *LPINIT)(IBurnModel* pModel, IBurnController* pController);

    public delegate HRESULT InitDelegate(IBurnModel* pModel, IBurnController* pController);

    //typedef HRESULT (CALLBACK *LPNOARGUMENTS)();

    //public delegate HRESULT NoArgumentsDelegate();

    typedef void (CALLBACK *LPNOARGUMENTSNORETURN)();

    public delegate void NoArgumentsNoReturnDelegate();

    typedef BOOL (CALLBACK *LPPROGRESS)(DWORD dwPercentage, DWORD dwOverallPercentage);

    public delegate bool ProgressDelegate(DWORD dwPercentage, DWORD dwOverallPercentage);

    typedef BOOL (CALLBACK *LPBEGIN)(DWORD cPackages);

    public delegate BOOL BeginDelegate(DWORD cPackages);

    typedef BOOL (CALLBACK *LPPACKAGEBEGIN)(IBurnPackage* pPackage);

    public delegate BOOL PackageBeginDelegate(IBurnPackage* pPackage);

    typedef BOOL (CALLBACK *LPCACHEBEGIN)(DWORD cCachingPackages, DWORD cTotalPayloads, DWORD64 dw64TotalCacheSize);

    public delegate BOOL CacheBeginDelegate(DWORD cCachingPackages, DWORD cTotalPayloads, DWORD64 dw64TotalCacheSize);

    typedef BOOL (CALLBACK *LPPACKAGEBEGINCACHE)(IBurnPackage* pPackage, DWORD cPayloads, DWORD64 dw64PackageCacheSize);

    public delegate BOOL PackageBeginCacheDelegate(IBurnPackage* pPackage, DWORD cPayloads, DWORD64 dw64PackageCacheSize);

    typedef BOOL (CALLBACK *LPPAYLOADBEGINCACHE)(IBurnPackage* pPackage, IBurnPayload* pPayload, SOURCE_TYPE sourceType, LPCWSTR wzSourcePath);

    public delegate BOOL PayloadBeginCacheDelegate(IBurnPackage* pPackage, IBurnPayload* pPayload, SOURCE_TYPE sourceType, LPCWSTR wzSourcePath);

    typedef BOOL (CALLBACK *LPPAYLOADPROGRESS)(IBurnPackage* pPackage, IBurnPayload* pPayload, DWORD64 dw64PayloadCached, DWORD64 dw64PayloadSize);

    public delegate BOOL PayloadProgressDelegate(IBurnPackage* pPackage, IBurnPayload* pPayload, DWORD64 dw64PayloadCached, DWORD64 dw64PayloadSize);

    typedef void (CALLBACK *LPPAYLOADCOMPLETE)(IBurnPackage* pPackage, IBurnPayload* pPayload, HRESULT hrStatus);

    public delegate void PayloadCompleteDelegate(IBurnPackage* pPackage, IBurnPayload* pPayload, HRESULT hrStatus);

    typedef BOOL (CALLBACK *LPPACKAGEBEGINEXECUTE)(IBurnPackage* pPackage, BOOL fExecute);

    public delegate BOOL PackageBeginExecuteDelegate(IBurnPackage* pPackage, BOOL fExecute);

    typedef void (CALLBACK *LPPACKAGECOMPLETE)(IBurnPackage* pPackage, HRESULT hrStatus);

    public delegate void PackageCompleteDelegate(IBurnPackage* pPackage, HRESULT hrStatus);

    typedef void (CALLBACK *LPCOMPLETE)(HRESULT hrStatus);

    public delegate void CompleteDelegate(HRESULT hrStatus);

    typedef INT (CALLBACK *LPERROR)(IBurnPackage* pPackage, DWORD dwCode, LPCWSTR wzMessage, UINT uiFlags);

    public delegate int ErrorDelegate(IBurnPackage* pPackage, DWORD dwCode, LPCWSTR wzMessage, UINT uiFlags);


    //
    // CBurnViewMockProxy - private class that translates native interface calls to managed delegate calls.
    //
    private class CBurnViewMockProxy : public IBurnView
    {
    public:
        // IUnknown - minimal implementation.
        STDMETHODIMP QueryInterface(
            __in const IID&,
            __out void**
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP_(ULONG) AddRef()
        {
            return ::InterlockedIncrement(&this->m_cReferences);
        }

        STDMETHODIMP_(ULONG) Release()
        {
            long l = ::InterlockedDecrement(&this->m_cReferences);
            if (0 < l)
            {
                return l;
            }

            delete this;
            return 0;
        }

        // IBurnView - proxy calls to managed mock.
        STDMETHODIMP Initialize(
            __in IBurnModel* pModel,
            __in IBurnController* pController
            )
        {
            return (*m_pfnInternalInit)(pModel, pController);
        }

        STDMETHODIMP_(void) Uninitialize()
        {
            (*m_pfnUninitialize)();
        }

        STDMETHODIMP Run()
        {
            return (*m_pfnRun)();
        }

        STDMETHODIMP_(BOOL) OnPrepareBegin(
            __in DWORD cPackages
            )
        {
            return (*m_pfnOnPrepareBegin)(cPackages);
        }

        STDMETHODIMP_(BOOL) OnPreparePackageBegin(
            __in IBurnPackage* pPackage
            )
        {
            return (*m_pfnOnPreparePackageBegin)(pPackage);
        }

        STDMETHODIMP_(void) OnPreparePackageComplete(
            __in IBurnPackage* pPackage,
            __in HRESULT hrStatus,
            __in PACKAGE_STATE state,
            __in REQUEST_STATE requested,
            __in ACTION_STATE execute,
            __in ACTION_STATE rollback
            )
        {
            (*m_pfnOnPreparePackageComplete)(pPackage, S_OK);;
        }

        STDMETHODIMP_(void) OnPrepareComplete(
            __in HRESULT hrStatus
            )
        {
            (*m_pfnOnPrepareComplete)(hrStatus);
        }

        STDMETHODIMP_(BOOL) OnCacheBegin(
            __in DWORD cCachingPackages,
            __in DWORD cTotalPayloads,
            __in DWORD64 dw64TotalCacheSize
            )
        {
            return (*m_pfnOnCacheBegin)(cCachingPackages, cTotalPayloads, dw64TotalCacheSize);
        }

        STDMETHODIMP_(BOOL) OnCachePackageBegin(
            __in IBurnPackage* pPackage,
            __in DWORD cPayloads,
            __in DWORD64 dw64PackageCacheSize
            )
        {
            return (*m_pfnOnCachePackageBegin)(pPackage, cPayloads, dw64PackageCacheSize);
        }

        STDMETHODIMP_(BOOL) OnCachePackagePayloadBegin(
            __in_opt IBurnPackage* pPackage,
            __in IBurnPayload* pPayload,
            __in SOURCE_TYPE sourceType,
            __in_z LPCWSTR wzSourcePath
            )
        {
            return (*m_pfnOnCachePackagePayloadBegin)(pPackage, pPayload, sourceType, wzSourcePath);
        }

        STDMETHODIMP_(BOOL) OnCacheProgress(
            __in_opt IBurnPackage* pPackage,
            __in IBurnPayload* pPayload,
            __in DWORD64 dw64PayloadCached,
            __in DWORD64 dw64PayloadSize
            )
        {
            return (*m_pfnOnCacheProgress)(pPackage, pPayload, dw64PayloadCached, dw64PayloadSize);
        }

        STDMETHODIMP_(void) OnCachePackagePayloadComplete(
            __in_opt IBurnPackage* pPackage,
            __in IBurnPayload* pPayload,
            __in HRESULT hrExitCode
            )
        {
            (*m_pfnOnCachePackagePayloadComplete)(pPackage, pPayload, hrExitCode);
        }

        STDMETHODIMP_(void) OnCachePackageComplete(
            __in IBurnPackage* pPackage,
            __in HRESULT hrExitCode
            )
        {
            (*m_pfnOnCachePackageComplete)(pPackage, hrExitCode);
        }

        STDMETHODIMP_(void) OnCacheComplete(
            __in HRESULT hrStatus
            )
        {
            (*m_pfnOnCacheComplete)(hrStatus);
        }

        STDMETHODIMP_(BOOL) OnExecuteBegin(
            __in DWORD cPackages
            )
        {
            return (*m_pfnOnExecuteBegin)(cPackages);
        }

        STDMETHODIMP_(BOOL) OnExecutePackageBegin(
            __in IBurnPackage* pPackage,
            __in BOOL fExecute
            )
        {
            return (*m_pfnOnExecutePackageBegin)(pPackage, fExecute);
        }

        STDMETHODIMP_(INT) OnExecuteMsiMessage(
            __in IBurnMsiPackage* pPackage,
            __in INSTALLMESSAGE mt,
            __in UINT uiFlags,
            __in LPCWSTR wzMessage,
            __in_opt MSIHANDLE hRecord
            )
        {
            return IDOK;
        }

        STDMETHODIMP_(BOOL) OnExecuteProgress(
            __in IBurnPackage* pPackage,
            __in DWORD dwPercentage
            )
        {
            DWORD dwOverallPercentage = dwPercentage;
            return (*m_pfnProgress)(dwPercentage, dwOverallPercentage);
        }

        STDMETHODIMP_(INT) OnExecuteError(
            __in IBurnPackage* pPackage,
            __in DWORD dwCode,
            __in_z LPCWSTR wzMessage,
            __in UINT uiFlags
            )
        {
            return (*m_pfnOnError)(pPackage, dwCode, wzMessage, uiFlags);
        }

        STDMETHODIMP_(void) OnExecutePackageComplete(
            __in IBurnPackage* pPackage,
            __in HRESULT hrStatus
            )
        {
            (*m_pfnOnExecutePackageComplete)(pPackage, hrStatus);
        }

        STDMETHODIMP_(void) OnExecuteComplete(
            __in HRESULT hrStatus
            )
        {
            (*m_pfnOnExecuteComplete)(hrStatus);
        }

        STDMETHODIMP_(BOOL) OnRestartRequired()
        {
             // Always suppress the restart request.
            return FALSE; // TODO: consider calling the Mock base if we ever trust the code not to restart.
        }

        STDMETHODIMP GetControlText(
            __in DWORD dwControlID,
            __out LPWSTR* psczText
            )
        {
            return S_FALSE;
        }

        // IBurnCore - not implemented
        STDMETHODIMP GetPackageCount(
            __out DWORD* pcPackages
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP GetCommandLineParameters(
        __out LPWSTR* psczCommandLine
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP SetPackageProperty(
            __in_z LPCWSTR wzPackageId,
            __in_z LPCWSTR wzProperty,
            __in_z_opt LPCWSTR wzValue
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP Prepare(
            __in_z_opt LPCWSTR wzPackageId,
            __in REQUEST_STATE requested
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP Cache(
            __in_z_opt LPCWSTR wzPackageId
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP Execute(
            __in REQUEST_STATE requested
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP WaitIfPaused()
        {
            return S_OK;
        }

    public:
        CBurnViewMockProxy() : m_cReferences(1)
        {
        }

    public:
        LPINIT m_pfnInternalInit;
        LPNOARGUMENTS m_pfnRun;
        LPNOARGUMENTSNORETURN m_pfnUninitialize;

        LPBEGIN m_pfnOnPrepareBegin;
        LPPACKAGEBEGIN m_pfnOnPreparePackageBegin;
        LPPACKAGECOMPLETE m_pfnOnPreparePackageComplete;
        LPCOMPLETE m_pfnOnPrepareComplete;

        LPCACHEBEGIN m_pfnOnCacheBegin;
        LPPACKAGEBEGINCACHE m_pfnOnCachePackageBegin;
        LPPAYLOADBEGINCACHE m_pfnOnCachePackagePayloadBegin;
        LPPAYLOADPROGRESS m_pfnOnCacheProgress;
        LPPAYLOADCOMPLETE m_pfnOnCachePackagePayloadComplete;
        LPPACKAGECOMPLETE m_pfnOnCachePackageComplete;
        LPCOMPLETE m_pfnOnCacheComplete;

        LPBEGIN m_pfnOnExecuteBegin;
        LPPACKAGEBEGINEXECUTE m_pfnOnExecutePackageBegin;
        LPPROGRESS m_pfnProgress;
        LPPACKAGECOMPLETE m_pfnOnExecutePackageComplete;
        LPCOMPLETE m_pfnOnExecuteComplete;

        LPERROR m_pfnOnError;

    private:
        long m_cReferences;
    };


    public ref class BurnViewMockBase : public IBurnViewShim
    {
    public: // IBurnViewShim
        virtual IBurnView* GetView()
        {
            m_pMockProxy->AddRef();
            return m_pMockProxy;
        }

    public:
        virtual HRESULT Initialize(BurnModelTestShim^ model, BurnControllerTestShim^ controller);

        virtual HRESULT Run();

        virtual void Uninitialize();

        virtual bool OnPrepareBegin(DWORD packageCount);

        virtual bool OnPreparePackageBegin(BurnPackageTestShim^ package);

        virtual void OnPreparePackageComplete(BurnPackageTestShim^ package, HRESULT status);

        virtual void OnPrepareComplete(HRESULT status);

        virtual bool OnCacheBegin(DWORD cCachingPackages, DWORD cTotalPayloads, DWORD64 dw64TotalCacheSize);

        virtual bool OnCachePackageBegin(BurnPackageTestShim^ package, DWORD cPayloads, DWORD64 dw64PackageCacheSize);

        virtual bool OnCachePackagePayloadBegin(BurnPackageTestShim^ package, BurnPayloadTestShim^ payload, PayloadSourceType sourceType, String^ wzSourcePath);

        virtual bool OnCachePackagePayloadProgress(BurnPackageTestShim^ package, BurnPayloadTestShim^ payload, DWORD64 dw64PayloadCached, DWORD64 dw64PayloadSize);

        virtual void OnCachePackagePayloadComplete(BurnPackageTestShim^ package, BurnPayloadTestShim^ payload, HRESULT hrStatus);

        virtual void OnCachePackageComplete(BurnPackageTestShim^ package, HRESULT hrStatus);

        virtual void OnCacheComplete(HRESULT status);

        virtual bool OnExecuteBegin(DWORD executingCount);

        virtual bool OnExecutePackageBegin(BurnPackageTestShim^ package, bool execute);

        virtual bool OnProgress(DWORD percentage, DWORD overallPercentage);

        virtual DialogResult OnError(DWORD dwCode, String^ message, MessageBoxButtons buttons, MessageBoxIcon icon);

        virtual void OnExecutePackageComplete(BurnPackageTestShim^ package, HRESULT status);

        virtual void OnExecuteComplete(HRESULT status);

    public:
        BurnViewMockBase();

        ~BurnViewMockBase();

    protected:
        !BurnViewMockBase();

    internal:
        HRESULT InternalInit(IBurnModel* pModel, IBurnController* pController);

        BOOL InternalOnPrepareBegin(DWORD cPackages);

        BOOL InternalOnPreparePackageBegin(IBurnPackage* pPackage);

        void InternalOnPreparePackageComplete(IBurnPackage* pPackage, HRESULT hrStatus);

        BOOL InternalOnCacheBegin(DWORD cCachingPackages, DWORD cTotalPayloads, DWORD64 dw64TotalCacheSize);

        BOOL InternalOnCachePackageBegin(IBurnPackage* pPackage, DWORD cPayloads, DWORD64 dw64PackageCacheSize);

        BOOL InternalOnCachePayloadBegin(IBurnPackage* pPackage, IBurnPayload* pPayload, SOURCE_TYPE sourceType, LPCWSTR wzSourcePath);

        BOOL InternalOnCachePayloadProgress(IBurnPackage* pPackage, IBurnPayload* pPayload, DWORD64 dw64PayloadCached, DWORD64 dw64PayloadSize);

        void InternalOnCachePayloadComplete(IBurnPackage* pPackage, IBurnPayload* pPayload, HRESULT hrStatus);

        void InternalOnCachePackageComplete(IBurnPackage* pPackage, HRESULT hrStatus);

        BOOL InternalOnExecuteBegin(DWORD cExecutingPackages);

        BOOL InternalOnExecutePackageBegin(IBurnPackage* pPackage, BOOL fExecute);

        void InternalOnExecutePackageComplete(IBurnPackage* pPackage, HRESULT hrStatus);

        INT InternalOnError(IBurnPackage* pPackage, DWORD dwCode, LPCWSTR wzMessage, UINT uiFlags);

    private:
        void Destroy();

    private:
        CBurnViewMockProxy* m_pMockProxy;

        InitDelegate^ init;
        NoArgumentsDelegate^ run;
        NoArgumentsNoReturnDelegate^ uninit;

        BeginDelegate^ onPrepareBegin;
        PackageBeginDelegate^ onPreparePackageBegin;
        PackageCompleteDelegate^ onPreparePackageComplete;
        CompleteDelegate^ onPrepareComplete;

        CacheBeginDelegate^ onCacheBegin;
        PackageBeginCacheDelegate^ onCachePackageBegin;

        PayloadBeginCacheDelegate^ onCachePackagePayloadBegin;
        PayloadProgressDelegate^ onCacheProgress;
        PayloadCompleteDelegate^ onCachePackagePayloadComplete;
        PackageCompleteDelegate^ onCachePackageComplete;
        CompleteDelegate^ onCacheComplete;

        BeginDelegate^ onExecuteBegin;
        PackageBeginExecuteDelegate^ onExecutePackageBegin;
        ProgressDelegate^ onExecuteProgress;
        PackageCompleteDelegate^ onExecutePackageComplete;
        CompleteDelegate^ onExecuteComplete;

        ErrorDelegate^ onError;
    };
}
}
}
}
}
