//-------------------------------------------------------------------------------------------------
// <copyright file="InstallationAbstractionLayerMockBase.cpp" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#include "precomp.h"

using namespace System::Runtime::InteropServices;


namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
//
// IsElevated - base do nothing call.
//
HRESULT InstallationAbstractionLayerMockBase::IsElevated()
{
    return E_NOTIMPL;
}


//
// ShellExecute - base do nothing call.
//
HRESULT InstallationAbstractionLayerMockBase::ShellExecute(
    String^ fileName,
    String^ parameters
    )
{
    return S_OK;
}


//
// InternalShellExecute - called to do ShellExecute().
//
HRESULT InstallationAbstractionLayerMockBase::InternalShellExecute(LPCWSTR wzFilename, LPCWSTR wzParameters)
{
    String^ fileName = gcnew String(wzFilename);
    String^ paramters = gcnew String(wzParameters);

    return this->ShellExecute(fileName, paramters);
}


//
// Constructor
//
InstallationAbstractionLayerMockBase::InstallationAbstractionLayerMockBase()
{
    this->m_pMockProxy = new CInstallationAbstractionLayerMockProxy();

    this->isElevated = gcnew NoArgumentsDelegate(this, &InstallationAbstractionLayerMockBase::IsElevated);
    this->m_pMockProxy->m_pfnIsElevated = static_cast<LPNOARGUMENTS>(Marshal::GetFunctionPointerForDelegate(this->isElevated).ToPointer());

    this->shellExecute = gcnew ShellExecuteDelegate(this, &InstallationAbstractionLayerMockBase::InternalShellExecute);
    this->m_pMockProxy->m_pfnShellExecute = static_cast<LPSHELLEXECUTE>(Marshal::GetFunctionPointerForDelegate(this->shellExecute).ToPointer());
}


//
// Destructor
//
InstallationAbstractionLayerMockBase::~InstallationAbstractionLayerMockBase()
{
    this->Destroy();
}


//
// Finalizer
//
InstallationAbstractionLayerMockBase::!InstallationAbstractionLayerMockBase()
{
    this->Destroy();
}


//
// Destroy - cleans up the object (called by destructor and finalizer).
//
void InstallationAbstractionLayerMockBase::Destroy()
{
    ReleaseNullObject(this->m_pMockProxy);
}
}
}
}
}
}
