//-------------------------------------------------------------------------------------------------
// <copyright file="InstallationAbstractionLayerMockBase.h" company="Microsoft">
//    Copyright (c) Microsoft Corporation.  All rights reserved.
//    
//    The use and distribution terms for this software are covered by the
//    Common Public License 1.0 (http://opensource.org/licenses/cpl.php)
//    which can be found in the file CPL.TXT at the root of this distribution.
//    By using this software in any fashion, you are agreeing to be bound by
//    the terms of this license.
//    
//    You must not remove this notice, or any other, from this software.
// </copyright>
//-------------------------------------------------------------------------------------------------

#pragma once

using namespace System::Windows::Forms;

namespace MS
{
namespace Tools
{
namespace WindowsInstallerXml
{
namespace Test
{
namespace Bootstrapper
{
    typedef HRESULT (CALLBACK *LPSHELLEXECUTE)(LPCWSTR wzFilename, LPCWSTR wzParameters);

    public delegate HRESULT ShellExecuteDelegate(LPCWSTR wzFliename, LPCWSTR wzParameters);

    typedef HRESULT (CALLBACK *LPNOARGUMENTS)();

    public delegate HRESULT NoArgumentsDelegate();


    //
    // CInstallationAbstractionLayerMockProxy - private class that translates native interface calls to managed delegate calls.
    //
    private class CInstallationAbstractionLayerMockProxy : public IInstallationAbstractionLayer
    {
    public:
        // IUnknown - minimal implementation.
        STDMETHODIMP QueryInterface(
            __in const IID&,
            __out void**
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP_(ULONG) AddRef()
        {
            return ::InterlockedIncrement(&this->m_cReferences);
        }

        STDMETHODIMP_(ULONG) Release()
        {
            long l = ::InterlockedDecrement(&this->m_cReferences);
            if (0 < l)
            {
                return l;
            }

            delete this;
            return 0;
        }

        STDMETHODIMP CreateProcess(
            __in LPCWSTR lpApplicationName,
            __inout LPWSTR lpCommandLine,
            __in LPSECURITY_ATTRIBUTES lpProcessAttributes,
            __in LPSECURITY_ATTRIBUTES lpThreadAttributes,
            __in BOOL bInheritHandles,
            __in DWORD dwCreationFlags,
            __in LPVOID lpEnvironment,
            __in LPCWSTR lpCurrentDirectory,
            __in LPSTARTUPINFOW lpStartupInfo,
            __out LPPROCESS_INFORMATION lpProcessInformation
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP GetExitCodeProcess(
            __in HANDLE hProcess,
            __out LPDWORD lpExitCode
            )
        {
            return E_NOTIMPL;
        }

        IFACEMETHODIMP IsElevated()
        {
            return (*m_pfnIsElevated)();
        }

        STDMETHODIMP MsiGetProductInfo(
            __in LPCWSTR szProduct,
            __in LPCWSTR szProperty,
            __out LPWSTR* psczValue
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP_(INSTALLSTATE) MsiQueryFeatureState(
            __in LPCWSTR szProduct,
            __in LPCWSTR szFeature
            )
        {
            return INSTALLSTATE_UNKNOWN;
        }


        STDMETHODIMP_(INSTALLUILEVEL) MsiSetInternalUI(
            __in INSTALLUILEVEL dwUILevel,
            __inout HWND* phWnd
            )
        {
            return INSTALLUILEVEL_DEFAULT;
        }


        STDMETHODIMP MsiSetExternalUI(
            __in INSTALLUI_HANDLERW puiHandler,
            __in DWORD dwMessageFilter,
            __in LPVOID pvContext,
            __out_opt INSTALLUI_HANDLERW* ppuiPrevHandler
            )
        {
            return E_NOTIMPL;
        }


        STDMETHODIMP MsiSetExternalUIRecord(
            __in INSTALLUI_HANDLER_RECORD puiHandler,
            __in DWORD dwMessageFilter,
            __in LPVOID pvContext,
            __out_opt PINSTALLUI_HANDLER_RECORD ppuiPrevHandler
            )
        {
            return E_NOTIMPL;
        }


        STDMETHODIMP MsiEnableLog(
            __in DWORD dwLogMode,
            __in LPCWSTR szLogFile,
            __in DWORD dwLogAttributes
            )
        {
            return E_NOTIMPL;
        }


        STDMETHODIMP MsiBeginTransaction(
            __in LPCWSTR szName,
            __in DWORD dwTransactionAttributes,
            __out MSIHANDLE* phTransactionHandle,
            __out HANDLE* phChangeOfOwnerEvent
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP MsiEndTransaction(
            __in DWORD dwTransactionState
            )
        {
            return E_NOTIMPL;
        }


        STDMETHODIMP MsiJoinTransaction(
            __in MSIHANDLE hTransactionID,
            __in DWORD dwTransactionAttributes,
            __out HANDLE* phChangeOfOwnerEvent
            )
        {
            return E_NOTIMPL;
        }


        STDMETHODIMP MsiInstallProduct(
            __in LPCWSTR szPackagePath,
            __in LPCWSTR szCommandLine
            )
        {
            return E_NOTIMPL;
        }


        STDMETHODIMP MsiConfigureProductEx(
            __in LPCWSTR szProduct,
            __in int iInstallLevel,
            __in INSTALLSTATE eInstallState,
            __in LPCWSTR szCommandLine
            )
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP ScheduleRestart()
        {
            return E_NOTIMPL;
        }

        STDMETHODIMP ShellExecute(
            __in SHELLEXECUTEINFOW* pInfo
            )
        {
            return (*m_pfnShellExecute)(pInfo->lpFile, pInfo->lpParameters);
        }

    public:
        CInstallationAbstractionLayerMockProxy() : m_cReferences(1)
        {
        }

    public:
        LPNOARGUMENTS m_pfnIsElevated;
        LPSHELLEXECUTE m_pfnShellExecute;

    private:
        long m_cReferences;
    };

    public ref class InstallationAbstractionLayerMockBase : public IInstallationAbstractionLayerShim
    {
    public: // IInstallationAbstractionLayerShim
        virtual IInstallationAbstractionLayer* GetInstallationAbstractionLayer()
        {
            m_pMockProxy->AddRef();
            return m_pMockProxy;
        }

    public:
        virtual HRESULT IsElevated();

        virtual HRESULT ShellExecute(String^ fileName, String^ parameters);

    public:
        InstallationAbstractionLayerMockBase();

        ~InstallationAbstractionLayerMockBase();

    protected:
        !InstallationAbstractionLayerMockBase();

    internal:
        HRESULT InternalShellExecute(LPCWSTR wzFilename, LPCWSTR wzParameters);

    private:
        void Destroy();

    private:
        CInstallationAbstractionLayerMockProxy* m_pMockProxy;

        NoArgumentsDelegate^ isElevated;
        ShellExecuteDelegate^ shellExecute;
    };
}
}
}
}
}
